        PROGRAM WEPS
        IMPLICIT NONE
C
        INTEGER NP,N
        PARAMETER (NP=1000)
 
        REAL*8 A(NP,NP),B(NP,NP),X(NP),Y(NP),V(NP)
        INTEGER INDX(NP)
 
        N=NP
 
        CALL TSTSOL(A,B,X,Y,V,INDX,N,NP)
 
        END
 
        SUBROUTINE TSTSOL(A,B,X,Y,V,INDX,N,NP)
        IMPLICIT none
        DIMENSION A(NP,N),B(NP,N),X(N),Y(N),V(N),INDX(N)
        real*8 a,b,x,y,v
        integer indx,n,np

        integer i,j,ier
        real*8 d,s

           call random_number(b)
           call random_number(y)
 
           DO 40 I=1,N
              DO 30 J=1,N
                 A(I,J)=B(I,J)
   30            CONTINUE
              X(I)=Y(I)
   40         CONTINUE
           CALL LUDCMP(A,N,NP,INDX,D,V,IER)
           IF (IER.NE.0) THEN
              PRINT *,' MATRIX SINGULAR IN TSTSOL'
              PRINT *,' PROGAM ENDS HERE'
              STOP
              ENDIF
           CALL LUBKSB(A,N,NP,INDX,X)
           DO 70  I=1,N
              S=0.0D0
              DO 60 J=1,N
                 S=S+B(I,J)*X(J)
   60            CONTINUE
              IF (ABS(S-Y(I)).GT.0.01D0*N) THEN
                 PRINT *,' INCORRECT SOLUTION IN TSTSOL'
                 PRINT *,' PROGRAM ENDS HERE'
                 STOP
                 ENDIF
   70         CONTINUE
 
        RETURN
        END
 
        SUBROUTINE LUDCMP(A,N,NP,INDX,D,VV,IER)
        IMPLICIT none
        real*8 A(NP,N),VV(N)
        integer n,np,indx(n),ier
        
c
c       see: Numerical Recipes, chapter 2.3
c
c       given an N * N matrix A, with physical dimensions NP, this routine
c       replaces it by the LU decomposition of a rowwise permutation of
c       itself. A and N are input. A is output, arranged as follows:
c       strict under triangle = L, upper triangle = U.
c       VV is a workarray with dimension of at least N.(this is a
c       minor deviation from the original text).
c       INDX is an output vector which records the row permutation
c       effected by the partial pivoting; D is output as +1 or -1
c       depending on whether the number of row interchanges was even
c       or odd, respectively. This routine is used in combination with
c       LUBKSB to solve linear equations or inver a matrix.
c       In case A is singular, IER=1 and D=0.0 further calculations are
c       bypassed.
c       In case A is nearly singular, IER =2.
c       IER=0 if all is ok.
c       The determinant of A is obtained by multiplying the product of the
c       main diagonal elements of the output matrix A with D.
c
        real*8 tiny, sum, dum,d,aamax
        integer k,imax,i,j
        PARAMETER (TINY=1.0D-20)
        D=1.0D0
        IER=0
C       LOOP OVER ROWS TO GET THE IMPLICIT SCALING INFORMATION
        imax=0
        DO 12 I=1,N
           AAMAX=0.0D0
           DO 11 J=1,N
              IF (ABS(A(I,J)).GT.AAMAX) AAMAX=ABS(A(I,J))
   11         CONTINUE
           IF (AAMAX.EQ.0.0D0) THEN
              IER=1
              D=0.0D0
              RETURN
              ENDIF
           VV(I)=1.0D0/AAMAX
   12      CONTINUE
C THIS IS THE LOOP OVER COLUMS OF CROUT'S METHOD
       DO 19 J=1,N
          IF (J.GT.1) THEN
             DO 14 I=1,J-1
                SUM=A(I,J)
                IF (I.GT.1) THEN
                   DO 13 K=1,I-1
                      SUM=SUM-A(I,K)*A(K,J)
   13                 CONTINUE
                   A(I,J)=SUM
                   ENDIF
   14           CONTINUE
             ENDIF
C INITIALIZE FOR THE SEARCH OF THE LARGEST PIVOT ELEMENT
          AAMAX=0.0D0
          DO 16 I=J,N
             SUM=A(I,J)
             IF (J.GT.1) THEN
                DO 15 K=1,J-1
                   SUM=SUM-A(I,K)*A(K,J)
   15              CONTINUE
                A(I,J)=SUM
                ENDIF
C FIGURE OF MERIT FOR THE PIVOT
             DUM=VV(I)*ABS(SUM)
             IF (DUM.GT.AAMAX) THEN
                IMAX=I
                AAMAX=DUM
                ENDIF
   16        CONTINUE
C INTERCHANGE ROWS IF NECESSARY
          IF (J.NE.IMAX) THEN
             DO 17 K=1,N
                DUM=A(IMAX,K)
                A(IMAX,K)=A(J,K)
                A(J,K)=DUM
   17           CONTINUE
             D=-D
             VV(IMAX)=VV(J)
             ENDIF
          INDX(J)=IMAX
          IF (J.NE.N) THEN
             IF (A(J,J).EQ.0) THEN
                A(J,J)=TINY
                IER=2
C IF THE PIVOT ELEMENT IS ZERO THE MATRIX IS SINGULAR (AT LEAST
C TO THE PRECISION OF THE ALGORITHM). FOR SOME APPLICATIONS
C ON SINGULAR MATRICES, IT IS DESIRABLE TO SUBSTITUTE TINY FOR ZERO
                ENDIF
             DUM=1.0D0/A(J,J)
             DO 18 I=J+1,N
                A(I,J)=A(I,J)*DUM
   18           CONTINUE
             ENDIF
   19     CONTINUE
        IF (A(N,N).EQ.0.0D0)A(N,N)=TINY
        RETURN
        END
 
        SUBROUTINE LUBKSB(A,N,NP,INDX,B)
        IMPLICIT none
        real*8 A(NP,N),B(N)
        integer n,np
        integer INDX(N)
c
c Solves the set of lineair equations A.X=B . Here  A is input, not
c as the matrix A but rather as its LU decomposition, determined
c by the routine LUDCMP. INDX is input as permutation vector returned
c by LUDCMP. B is input as the right-hand side vector B and returns with
c the solution vector X. A,N,NP and INDX are not modified by this routine
c and can be left in place for successive calls with different right-hand
c sides B. This routine takes into account the possibility, that B will
c with many zero's, so it is efficient for use in matrix inversion.
c
        real*8 sum
        integer j,ll,ii,i
        II=0
        DO 12 I=1,N
           LL=INDX(I)
           SUM=B(LL)
           B(LL)=B(I)
           IF (II.NE.0) THEN
              DO 11 J=II,I-1
                 SUM=SUM-A(I,J)*B(J)
   11            CONTINUE
           ELSE IF (SUM.NE.0.0D0) THEN
              II=I
              ENDIF
           B(I)=SUM
   12      CONTINUE
        DO 14 I=N,1,-1
           SUM=B(I)
           IF (I.LT.N) THEN
              DO 13 J=I+1,N
                 SUM=SUM-A(I,J)*B(J)
   13            CONTINUE
              ENDIF
           B(I)=SUM/A(I,I)
   14      CONTINUE
        RETURN
        END
 
 
 
