use std::time::{SystemTime, UNIX_EPOCH};
use std::io;
const NP:usize = 2003;
const NR:usize = NP*NP;
const A1:f64 =1.0;
const A2:f64 =3.0;
const A3:f64 =3.0;

const TEPS:f64 = 0.1;

struct STATE1{iff:i32, ix1:i32, ix2:i32, ix3:i32, r:[f64;98],}
struct STATE2{iy:i32, iff:i32, jdum:i32, ir:[i32;98], }

fn ran1(idum:i32, state: &mut STATE1)->f64 {
    const M1:i32 = 259200;
    const IA1:i32 = 7141;
    const IC1:i32 = 54773;
    const RM1:f64 = 1.0/(M1 as f64);
    const M2:i32 = 134456;
    const IA2:i32 = 8121;
    const IC2:i32 = 28411;
    const RM2:f64 = 1.0/(M2 as f64);
    const M3:i32 = 243000;
    const IA3:i32 = 4561;
    const IC3:i32 = 51349;

    if idum < 0 || state.iff == 0
    {
        state.iff = 1;
        state.ix1=((IC1-idum)%M1).abs();
        state.ix1=(IA1*state.ix1+IC1) % M1;
        state.ix2 = state.ix1 % M2;
        state.ix1 = (IA1*state.ix1+IC1) % M1;
        state.ix3 = state.ix1 % M3;

        for j in 1..98
        {
            state.ix1 = (IA1*state.ix1+IC1) % M1;
            state.ix2 = (IA2*state.ix2+IC2) % M2;
            state.r[j] = ((state.ix1 as f64) + RM2*(state.ix2 as f64))*RM1;
        }
    }
    state.ix1 = (IA1*state.ix1+IC1) % M1;
    state.ix2 = (IA2*state.ix2+IC2) % M2;
    state.ix3 = (IA3*state.ix3+IC3) % M3;

    let j:usize = (1+(97*state.ix3)/M3) as usize;
    if j > 97 || j < 1
    {
        println!("THIS SHOULD NOT HAPPEN !!!, ROUTINE RAN1 J:{}",j);
        std::process::exit(1);
    }
    let x = state.r[j];
    state.r[j] = ((state.ix1 as f64)+RM2*(state.ix2 as f64))*RM1;
    x
}


fn ran2(idum:i32, state: &mut STATE2) -> f64 {
    const M:i32 = 714025;
    const IA:i32 = 1366;
    const IC:i32 = 150889;
    const RM:f64 = 1.0/(M as f64);
    if idum < 0 || state.iff == 0 {
        state.iff = 1;
        state.jdum = ((IC-idum) % M).abs();
        //for (j=1; j<=97; j++)
        for j in 1..98 { 
            state.jdum = (IA*state.jdum+IC) % M;
            state.ir[j] = state.jdum;
        }
        state.jdum = (IA*state.jdum+IC) % M;
        state.iy = state.jdum;
    }
    let j = 1+(97*state.iy)/M;
    if j > 97 || j < 1 {
        println!("THIS SHOULD NOT HAPPEN !!!, FUNCTION RAN2");
        println!("J={}",j);
        std::process::exit(1);
    }
    state.iy = state.ir[j as usize];
    let x = (state.iy as f64)*RM;
    state.jdum = (IA*state.jdum+IC) % M;
    state.ir[j as usize] = state.jdum;
    return x;
}

fn walltime() -> f64 {
    1.0e-6*(SystemTime::now()
            .duration_since(UNIX_EPOCH)
            .unwrap()
            .as_micros()
            as f64
           )
}

fn cptime() -> f64 {
    walltime()
}

fn dettmi()->f64 {
    let mut t0 = cptime();

    let m = 10;
    let p = 100.0;
    let mut dt = 0.0;
    let mut tmin = 0.0;

    let mut t = t0;
    for _i in 0..m { //(i=0; i<m; i++)
        {
            while t == t0 {
                t = cptime();
            }
            dt += t - t0;
            t0 = t;
        }

        tmin = p*dt/(m as f64);

        if tmin > 100.0 {
            println!("MINIMUM TIME FOR MEASUREMENTS GREATER THAN 100 SEC");
            println!("SOMETHING IS PROBABLY WRONG, PROGRAM STOPS");
            std::process::exit(1);
        }
    }
    tmin
}
fn meatcp(tmin:f64)->f64 {
    let mut l = 0;
    let t0 = cptime();
    let mut tcp = (l as f64)*t0;

    let mut dt = 0.0;
    while dt < tmin {
        l += 1;
        let t = cptime();
        dt = t - t0 - (l as f64)*tcp;
    }
    tcp = tmin/(l as f64);
    tcp
}

fn max(x:f64, y:f64)->f64{
    if x > y {
        x
    }
    else
    {
        y
    }
}

fn tstran(a: &mut[f64],n:usize, np:usize, tcp:f64, tmin:f64, state1: &mut STATE1)->f64 {
    let mut l = 0;
    let t0 = cptime();
    let mut dt = 0.0;
    let mut t;
    while dt < tmin {
        l += 1;
        for i in 0..n{
            for j in 0..n{
                a[(i + np*j) as usize] = ran1(1, state1);
            }
        }
        t = cptime();
        dt = t - t0 - (l as f64)*tcp;
    }
    t = dt/(l as f64);
    t
}


fn tstsol(a: &mut[f64], b: &mut[f64], x: &mut[f64],
          y: &mut[f64], v: &mut[f64], indx: &mut[usize], 
          n:usize, np:usize, tcp:f64, tmin:f64, state2: &mut STATE2) -> f64 {
    for i in 0..n as usize{
        for j in 0..n as usize {
            b[i+np*j] = ran2(1, state2);
        }
        y[i] = ran2(1, state2);
    }

    let mut l = 0;
    let mut dt = 0.0;
    let t0 = cptime();
    while dt < tmin {
        l += 1;
        for i in 0..n {
            for j in 0..n {
                a[i+np*j] = b[i+np*j];
            }
            x[i] = y[i];
        }
        let mut d:f64 = 0.0;
        let ier = ludcmp(a,n,np,indx,&mut d, v);
        if ier != 0 {
            println!("MATRIX SINGULAR IN TSTSOL");
            println!("PROGAM ENDS HERE");
            std::process::exit(1);
        }
        lubksb(a,n,np,indx,x);
        for i in 0..n {
            let mut s=0.0;
            for j in 0..n {
                s += b[i+np*j]*x[j];
            }
            if (s-y[i]).abs() > 0.01*(n as f64) {
                println!("INCORRECT SOLUTION IN TSTSOL");
                println!("PROGAM ENDS HERE");
                std::process::exit(1);
            }
        }
        let t = cptime();
        dt = t - t0 - (l as f64)*tcp;
    }
    dt/(l as f64)
}

fn ludcmp(a: &mut[f64], n:usize, np:usize, indx: &mut[usize],
          d: &mut f64, vv: &mut[f64]) -> i32{
    const TINY:f64 = 1.0e-20;
    *d = 1.0;

    let mut ier = 0;
    let mut imax = 10000000;
    let mut aamax:f64;

    for i in 0..n {
        aamax = 0.0;
        for j in 0..n { 
            if a[i+np*j].abs() > aamax{
                aamax = a[i+np*j].abs();
            }
        }
        if aamax == 0.0 {
            ier = 1;
            *d = 0.0;
            return ier;
        }
        vv[i] = 1.0/aamax;
    }
    for j in 0..n {
        if j > 0 {
            for i in 0..j {
                let mut sum = a[i+np*j];
                if i > 0 {
                    for k in 0..i{
                        sum -= a[i+np*k]*a[k+np*j];
                    }
                    a[i+np*j] = sum;
                }
            }
        }
        aamax = 0.0;
        for i in j..n {
            let mut sum = a[i+np*j];
            if  j > 0 {
                for k in 0..j{
                    sum -= a[i+np*k]*a[k+np*j];
                }
                a[i+np*j] = sum;
            }
            let dum = vv[i]*sum.abs();
            if dum > aamax {
                imax = i;
                aamax = dum;
            }
        }
        if j != imax {
            for k in 0..n {
                let dum = a[imax+np*k];
                a[imax+np*k] = a[j+np*k];
                a[j+np*k] = dum;
            }
            *d = -*d;
            vv[imax] = vv[j];
        }
        indx[j] = imax as usize;
        if j != n-1 {
            if a[j+np*j] == 0.0 {
                a[j+np*j] = TINY;
                ier = 2;
            }
            let dum = 1.0/a[j+np*j];
            for  i in j+1..n {
                a[i+np*j] = a[i+np*j]*dum;
            }
        }
    }
    if a[n-1+np*(n-1)] == 0.0 {
        a[n-1+np*(n-1)] = TINY;
    }
    return ier;
}

fn lubksb(a: &mut[f64], n:usize, np:usize, indx: &mut[usize], 
          b: &mut[f64]) {
    let mut ii=1234567;
    for i in 0..n {
        let ll = indx[i];
        let mut sum = b[ll];
        b[ll] = b[i];
        if ii != 1234567 {
            for j in ii..i{
                sum -= a[i+np*j]*b[j];
            }
        }
        else {
            if sum != 0.0 {
                ii = i;
            }
        }
        b[i] = sum;
    }
    for i in (0..n).rev() //for (i=n-1; i>=0; i--)
    {
        let mut sum = b[i];
        if  i < n-1 {
            for j in i+1..n {
                sum -= a[i+np*j]*b[j];
            }
        }
        b[i] = sum/a[i+np*i];
    }
    return;
}

fn tstsor(a: &mut[f64], n:usize , tcp:f64, tmin:f64, state2: &mut STATE2) -> f64
{
    for i in 0..n {
        a[i] = ran2(1,state2);
    }

    sort(a,n);
    let mut t:f64;
    let mut l = 0;
    let t0 = cptime();

    let mut dt = 0.0;

    while dt < tmin {
        l += 1;
        sort(a,n);

        t = cptime();
        dt=t - t0 -(l as f64)*tcp;
    }
    t = dt/(l as f64);

    let mut lll=true;

    for i in 0..n-1 {
        lll = lll && (a[i] <= a[i+1]);
    }

    if !lll {
        println!("ARRAY NOT SORTED IN TSTSOR");
        println!("PROGRAM ENDS HERE");
        std::process::exit(1);
    }

    return t;
}


fn sort(ra: &mut[f64], n:usize) {
    let mut l=n/2;
    let mut ir = n-1;
    let mut rra:f64;

    loop {
        if l > 0 {
            l -= 1;
            rra=ra[l];
        }
        else {
            rra = ra[ir];
            ra[ir] = ra[0];
            ir -= 1;
            if ir == 0 {
                ra[0] = rra;
                return;
            }
        }
        let mut i = l;
        let mut j = l+l+1;
        while j <= ir {
            if  j < ir {
                if  ra[j] < ra[j+1] {
                    j += 1;
                }
            }
            if rra < ra[j] {
                ra[i] = ra[j];
                i = j;
                j = j+j+1;
            }
            else {
                j = ir+1;
            }
        }
        ra[i] = rra;
    }
}


fn main() {
    println!("THIS IS PROGRAM WEPS: MEASURES CPU SPEED");
    println!("RUST VERSION");

    let mut tmin:f64 = -1.0;
    let mut tcp = 0.0;
    let mut a = vec![0.0;(NP*NP) as usize];
    let mut b = vec![0.0;(NP*NP) as usize];
    let mut x = vec![0.0;NP as usize];
    let mut y = vec![0.0;NP as usize];
    let mut v = vec![0.0;NP as usize];
    let mut aa = vec![0.0;NR as usize];
    let mut indx = vec![0 as usize;NP as usize];
    let mut s1 = STATE1{iff:0,ix1:0,ix2:0,ix3:0,r:[0.0;98]};
    ran1(-1,&mut s1);

    let mut s2 = STATE2{iy:0, iff:0, jdum:0, ir:[0;98]};
    ran2(-1,&mut s2);

    loop
    {
        println!("ENTER ORDER OF MATRICES TO BE USED, 0 TO STOP");
        let mut n = String::new();
        io::stdin()
            .read_line(&mut n)
            .expect("Failed to read line");
        let n:usize = n.trim().parse().expect("Please type a number!");
        if n == 0 {
            break;
        }
        if n > NP || n <= 1 {
            println!("ORDER MUST BE IN THE RANGE 2..{}",NP);
            continue;
        }
        if tmin <= 0.0 {
            tmin = dettmi();
            tcp = meatcp(tmin);
            tmin = max(max(100.0*tcp,tmin),1.0);
            println!("MINIMUM CPU TIME PER MEASUREMENT IS {} SEC",tmin);
        }
        let nsort = n*n;
        println!("NUMBER CRUNCHING ...");
        let t1 = tstran(&mut a,n,NP,tcp,tmin,&mut s1);
        let t2 = tstsol(&mut a,&mut b, &mut x, &mut y, &mut v, &mut indx, n, NP, tcp, tmin, &mut s2);
        let t3 = tstsor(&mut aa,nsort,tcp,tmin,&mut s2);

        let mut mark1 = ' ';
        let mut mark2 = ' ';
        let mut mark3 = ' ';

        if tcp > TEPS*t1 {
            mark1 = '*';
        }
        if tcp > TEPS*t2 {
            mark2 = '*';
        }
        if tcp > TEPS*t3 {
            mark3 = '*';
        }

        let inac = mark1 != ' ' || mark2 != ' ' || mark3 != ' ';

        let an = n as f64;

        let c1 = 10.0*an*an;
        let c2 = 1.0/3.0*(an*an*an+11.0*an*an);
        let c3 = (nsort as f64)*(nsort as f64).ln();

        let w1 = c1/t1*0.001;
        let w2 = c2/t2*0.001;
        let w3 = c3/t3*0.001;

        println!("DECLARED ORDER OF MATRICES: {}",NP);
        println!("ORDER OF MATRICES:          {}",n);
        println!("DIMENSION OF SORT ARRAY:    {}",nsort);
        let hw = (A1+A2+A3)/(A1/w1+A2/w2+A3/w3);
        print!("                                   ");
        print!("WEIGHT    CPUTIME");
        println!("         SPEED");
        print!("                                             ");
        print!("(SECONDS)");
        println!("      (KILOWEPS)");
        println!("");

        println!("SUBROUTINE, # CALLS IS {:9}:{:7.1}{:12.6}{:14.1}    {:1}",n*n,A1,t1,w1,mark1);
        println!("LIN EQUATIONS OF ORDER {:9}:{:7.1}{:12.6}{:14.1}    {:1}",n,A2,t2,w2,mark2);
        println!("SORTING, # ELEMENTS IS {:9}:{:7.1}{:12.6}{:14.1}    {:1}",n*n,A3,t3,w3,mark3);
        println!("HARMONIC WEIGHTED MEAN          :                   {:14.1}    {:1}",hw,mark3);

        if inac {
            println!(" * MEANS: POSSIBLY INACCURATE");
        }
        println!(" END OF THIS WEPS MEASUREMENT");
    }

    println!("END OF PROGRAM WEPS");
}
