        PROGRAM WEPS
        IMPLICIT NONE
C
C author: Willem Vermin
C         SARA
C         Kruislaan 415
C         1098SJ Amsterdam
C         the Netherlands
C
C
C e-mail: SONDWIM@HASARA11
C      or WILLEM_VERMIN@SARA.NL
C
C
C
C for IBM mainframes: compile with AUTODBL(DBL4) option
C
C  determines approximately cpu speed.
C
C  To adapt program to different operating systems: adapt
C  function CPTIME(), which must return the cpu-time elapsed,
C  measured in seconds.
C
C  Output is the used cptime and the number of Kilo WEPS
C  (an arbitrary unit) for each case. Furthermore, a weighted
C  mean and a weighted harmonic mean are output.
C
C  Three cases:
C
C -1 fill matrix A(NP,NP) with random numbers: exercizing
C    subroutine calls and simple calculations.
C    Weight: 1.0
C
C -2 solution of linair equations, matrix of coefficients:  A(NP,NP)
C    Weight: 3.0
C
C -3 sorting an array AA(NR)
C    Weight: 3.0
C
C  NP is order of matrix A, NR is length of array AA
C
C NP is maximum dimension of matrix to be used. Use larger value
C    for fast computers.
C
        INTEGER NP,NR,NSORT,N
        REAL*8 A1,A2,A3,HW,DUMMY,C1,C2,C3,AN
        REAL*8 W1,W2,W3,TEPS
        PARAMETER (NP=2003,NR=NP*NP)
C
C Weights for three cases:
C
        PARAMETER(A1=1.0D0,A2=3.0D0,A3=3.0D0)
 
C FACTOR USED FOR PRINTING WARNING ABOUT IMPRECISE RESULTS
 
        PARAMETER(TEPS=0.1D0)
 
C equivalence for computers with restricted memory:
        EQUIVALENCE (A,AA)
        REAL*8 A(NP,NP),B(NP,NP),X(NP),Y(NP),V(NP),AA(NR)
        INTEGER INDX(NP),LOOP1,LOOP2,LOOP3
        REAL*8 T1,T2,T3,TCP,TMIN,RAN1,RAN2
        CHARACTER MARK1,MARK2,MARK3,MARKH
        LOGICAL INAC
        DATA TMIN /-1.0D0/
 
        N=NP
 
        DUMMY=RAN1(-1)
        DUMMY=RAN2(-1)

! to quiet ftnchek
        DUMMY = DUMMY+DUMMY
 
        PRINT *,' THIS IS PROGRAM WEPS: MEASURES CPU SPEED'
        PRINT *,' FORTRAN VERSION'
    5   CONTINUE
        PRINT *
        PRINT *,' ENTER ORDER OF MATRICES TO BE USED, 0 TO STOP'
        READ (*,*,end=9000) N
        AN=N
        IF (N.EQ.0) GOTO 9000
        IF (N.GT.NP .OR. N.LT.0 .OR. N.EQ.1 ) THEN
           PRINT *,' ORDER MUST BE IN THE RANGE 2..' ,NP
           GOTO 5
           ENDIF
 
        CONTINUE
 
C DETERMINE  MINIMUM TIME PER MEASUREMENT:
        IF (TMIN .LT. 0) THEN
 
           CALL DETTMI(TMIN)
 
C MEASURE APPROXIMATE TIME (TCP) TO MEASURE CPTIME:
 
           CALL MEATCP(TCP,TMIN)
 
C IF NECCESARY, ADJUST TMIN TO BE AT LEAST 100 TIMES TCP 
 
           TMIN=MAX(100.0D0*TCP,TMIN,1.0D0)
 
           PRINT 8,TMIN
    8   FORMAT(' MINIMUM CPU TIME PER MEASUREMENT IS ',E12.4,' SEC')
        ENDIF
 
        NSORT=N*N
 
        PRINT *,' NUMBER CRUNCHING ...'
 
        CALL TSTRAN(A,N,NP,T1,LOOP1,TCP,TMIN)
        CALL TSTSOL(A,B,X,Y,V,INDX,N,NP,T2,LOOP2,TCP,TMIN)
        CALL TSTSOR(AA,NSORT,T3,LOOP3,TCP,TMIN)
 
C DETERMINE IF RESULTS ARE POSSIBLY INACCURATE DUE TO OVERHEAD
C OF MONITORING CPTIME:
 
        MARK1=' '
        MARK2=' '
        MARK3=' '
        MARKH=' '
 
        IF (TCP .GT. TEPS*T1) MARK1='*'
        IF (TCP .GT. TEPS*T2) MARK2='*'
        IF (TCP .GT. TEPS*T3) MARK3='*'
 
        INAC=(MARK1 .NE. ' ' .OR. MARK2 .NE. ' ' .OR. MARK3 .NE. ' ')
 
        IF (INAC) THEN
          MARKH='*'
        ENDIF
 
C C1, C2 AND C3 ARE PROPORTIONAL TO NUMBER OF CALCULATIONS IN
C  EACH CASE
 
        C1=10*AN**2
        C2=1.0D0/3*(AN**3+11*AN**2)
        C3=NSORT*DLOG(DBLE(NSORT))
 
C       CALCULATE KILOWEPS IN EACH CASE
 
        W1=C1/T1*1.0D-3
        W2=C2/T2*1.0D-3
        W3=C3/T3*1.0D-3
 
        PRINT 901,' DECLARED ORDER OF MATRICES ',NP
        PRINT 901,' ORDER OF MATRICES:         ',N
        PRINT 901,' DIMENSION OF SORT ARRAY:   ',NSORT
  901   FORMAT(A,I8)
 
C       CALCULATE WEIGHTED HARMONIC MEAN
 
        HW=(A1+A2+A3)/(A1/W1+A2/W2+A3/W3)
 
        PRINT 990
  990   FORMAT(/,30X,' WEIGHT    CPUTIME',
     +           '     REPS',
     +           '     SPEED',
     +         /,40X,'(SECONDS)',
     +           '           (KILOWEPS)')
        PRINT 1000,N*N,  A1,T1,LOOP1,W1,MARK1,
     +             N,    A2,T2,LOOP2,W2,MARK2,
     +             NSORT,A3,T3,LOOP3,W3,MARK3,
     +             HW,MARKH
 
 1000   FORMAT(/,' SUBROUTINE, # CALLS IS',I9,':',
     +                                     F4.1,F12.6,I8,F12.1,4X,A1,
     +         /,' LIN EQUATIONS OF ORDER',I9,':',
     +                                     F4.1,F12.6,I8,F12.1,4X,A1,
     +         /,' SORTING, # ELEMENTS IS',I9,':',
     +                                     F4.1,F12.6,I8,F12.1,4X,A1,
     +         /,' HARMONIC WEIGHTED MEAN',9X,':',
     +                                     4X,  20X  ,F12.1,4X,A1)
 
        IF(INAC)
     +  PRINT *,' * MEANS: POSSIBLY INACCURATE'
 
 
        PRINT *
        PRINT *,'  END OF THIS WEPS MEASUREMENT'
        GOTO 5
 
 9000   CONTINUE
        PRINT *,' END OF PROGRAM WEPS'
        END
 
        SUBROUTINE MEATCP(TCP,TMIN)
        IMPLICIT NONE
        REAL*8 TCP,T,T0,CPTIME,TMIN,DT
        INTEGER LOOP
 
        LOOP=0
        T0=CPTIME()
 
C MAKE TCP=0, WITHOUT LETTING THE COMPILER KNOW:
 
        TCP=LOOP*T0
 
   10   CONTINUE
 
        LOOP=LOOP+1
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        IF(DT .LT. TMIN) GOTO 10
 
        TCP=TMIN/LOOP
 
        RETURN
        END
 
        SUBROUTINE DETTMI(TMIN)
        IMPLICIT NONE
        REAL*8 TMIN,T,DT,T0,P,CPTIME
        INTEGER M,I
        PARAMETER (M=10,P=100.0D0)
 
C DETERMINES GRANULARITY OF TIME MEASUREMENT, AND CALCULATES
C TMIN, SO THAT TMIN IS A REASONABLE MINIMUM CPTIME FOR AN ACCURACY
C OF APPROXIMATELY ONE PERCENT
 
        T0=CPTIME()
        DT=0.0D0
        DO 100 I=1,M
   20   CONTINUE
        T=CPTIME()
        IF(T.EQ.T0) GOTO 20
        DT=DT+T-T0
        T0=T
  100   CONTINUE
 
        TMIN=P*DT/M
 
        IF(TMIN.GT.100.0D0) THEN
          PRINT *,' MINIMUM TIME FOR MEASUREMENTS GREATER THAN 100 SEC'
          PRINT *,' SOMETHING IS PROBABLY WRONG, PROGRAM STOPS'
          STOP
        ENDIF
        RETURN
        END
 
        SUBROUTINE TSTRAN(A,N,NP,T,LOOP,TCP,TMIN)
        IMPLICIT NONE
        REAL*8 A(NP,N)
        REAL*8 CPTIME,T,T0,TCP,TMIN,DT
        INTEGER N,NP
        REAL*8 RAN1
        INTEGER LOOP,I,J
 
        LOOP=0
        T0=CPTIME()
   10   CONTINUE
        LOOP=LOOP+1
 
           DO 90 I=1,N
              DO 80 J=1,N
                 A(I,J)=RAN1(1)
   80         CONTINUE
   90      CONTINUE
 
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        IF(DT .LT. TMIN) GOTO 10
 
        T=DT/LOOP
 
        RETURN
        END
 
        SUBROUTINE TSTSOL(A,B,X,Y,V,INDX,N,NP,T,LOOP,TCP,TMIN)
        IMPLICIT NONE
        INTEGER N,NP
        REAL*8 A(NP,N),B(NP,N),X(N),Y(N),V(N)
        INTEGER INDX(N)
        REAL*8 CPTIME,T,T0,TCP,TMIN,DT
        REAL*8 RAN2,S,D
        INTEGER LOOP
        INTEGER I,J,IER
 
           DO 20 I=1,N
              DO 10 J=1,N
                 B(I,J)=RAN2(1)
   10            CONTINUE
                 Y(I)=RAN2(1)
   20         CONTINUE
 
        LOOP=0
        T0=CPTIME()
   25   CONTINUE
        LOOP=LOOP+1
 
           DO 40 I=1,N
              DO 30 J=1,N
                 A(I,J)=B(I,J)
   30            CONTINUE
              X(I)=Y(I)
   40         CONTINUE
           CALL LUDCMP(A,N,NP,INDX,D,V,IER)
           IF (IER.NE.0) THEN
              PRINT *,' MATRIX SINGULAR IN TSTSOL'
              PRINT *,' PROGAM ENDS HERE'
              STOP
              ENDIF
           CALL LUBKSB(A,N,NP,INDX,X)
           DO 70  I=1,N
              S=0.0D0
              DO 60 J=1,N
                 S=S+B(I,J)*X(J)
   60            CONTINUE
              IF (ABS(S-Y(I)).GT.0.01D0*N) THEN
                 PRINT *,' INCORRECT SOLUTION IN TSTSOL'
                 PRINT *,' PROGRAM ENDS HERE'
                 STOP
                 ENDIF
   70         CONTINUE
 
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        IF(DT .LT. TMIN) GOTO 25
 
        T=DT/LOOP
 
        RETURN
        END
 
        SUBROUTINE TSTSOR(A,N,T,LOOP,TCP,TMIN)
        IMPLICIT NONE
        REAL*8 A(N)
        INTEGER N
        REAL*8 CPTIME,T,T0,TCP,TMIN,DT
        LOGICAL LLL
        REAL*8 RAN2
        INTEGER I,LOOP

        DO 10 I=1,N
           A(I)=RAN2(1)
  10       CONTINUE
 
C SORTING, SO ALL SORTS ARE EQUAL LATER ON
        CALL SORT(A,N)
        LOOP=0
        T0=CPTIME()
 
   15   CONTINUE
 
        LOOP=LOOP+1
 
C SORT USES HEAPSORT ALGORITHM: IT MAKES NO DIFFERENCE IF ARRAY IS
C ALREADY SORTED
 
        CALL SORT(A,N)
 
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        IF(DT .LT. TMIN) GOTO 15
 
        T=DT/LOOP
 
        LLL=.TRUE.
        DO 20 I=1,N-1
           LLL=LLL .AND. (A(I).LE.A(I+1))
   20      CONTINUE
        IF (.NOT.LLL)THEN
           PRINT *,' ARRAY NOT SORTED IN TSTSOR'
           PRINT *,' PROGRAM ENDS HERE'
           STOP
           ENDIF
 
        RETURN
        END
 
        SUBROUTINE LUDCMP(A,N,NP,INDX,D,VV,IER)
        IMPLICIT NONE
c
c       see: Numerical Recipes, chapter 2.3
c
c       given an N * N matrix A, with physical dimensions NP, this routine
c       replaces it by the LU decomposition of a rowwise permutation of
c       itself. A and N are input. A is output, arranged as follows:
c       strict under triangle = L, upper triangle = U.
c       VV is a workarray with dimension of at least N.(this is a
c       minor deviation from the original text).
c       INDX is an output vector which records the row permutation
c       effected by the partial pivoting; D is output as +1 or -1
c       depending on whether the number of row interchanges was even
c       or odd, respectively. This routine is used in combination with
c       LUBKSB to solve linear equations or inver a matrix.
c       In case A is singular, IER=1 and D=0.0 further calculations are
c       bypassed.
c       In case A is nearly singular, IER =2.
c       IER=0 if all is ok.
c       The determinant of A is obtained by multiplying the product of the
c       main diagonal elements of the output matrix A with D.
c
        INTEGER N,NP
        REAL*8 A(NP,N),VV(N),D
        INTEGER INDX(N),IER
        REAL*8 TINY,SUM,DUM,AAMAX
        PARAMETER (TINY=1.0D-20)
        INTEGER I,J,K,IMAX

        IMAX=0
        D=1.0D0
        IER=0
C       LOOP OVER ROWS TO GET THE IMPLICIT SCALING INFORMATION
        DO 12 I=1,N
           AAMAX=0.0D0
           DO 11 J=1,N
              IF (ABS(A(I,J)).GT.AAMAX) AAMAX=ABS(A(I,J))
   11         CONTINUE
           IF (AAMAX.EQ.0.0D0) THEN
              IER=1
              D=0.0D0
              RETURN
              ENDIF
           VV(I)=1.0D0/AAMAX
   12      CONTINUE
C THIS IS THE LOOP OVER COLUMS OF CROUT'S METHOD
       DO 19 J=1,N
          IF (J.GT.1) THEN
             DO 14 I=1,J-1
                SUM=A(I,J)
                IF (I.GT.1) THEN
                   DO 13 K=1,I-1
                      SUM=SUM-A(I,K)*A(K,J)
   13                 CONTINUE
                   A(I,J)=SUM
                   ENDIF
   14           CONTINUE
             ENDIF
C INITIALIZE FOR THE SEARCH OF THE LARGEST PIVOT ELEMENT
          AAMAX=0.0D0
          DO 16 I=J,N
             SUM=A(I,J)
             IF (J.GT.1) THEN
                DO 15 K=1,J-1
                   SUM=SUM-A(I,K)*A(K,J)
   15              CONTINUE
                A(I,J)=SUM
                ENDIF
C FIGURE OF MERIT FOR THE PIVOT
             DUM=VV(I)*ABS(SUM)
             IF (DUM.GT.AAMAX) THEN
                IMAX=I
                AAMAX=DUM
                ENDIF
   16        CONTINUE
C INTERCHANGE ROWS IF NECESSARY
          IF (J.NE.IMAX) THEN
             DO 17 K=1,N
                DUM=A(IMAX,K)
                A(IMAX,K)=A(J,K)
                A(J,K)=DUM
   17           CONTINUE
             D=-D
             VV(IMAX)=VV(J)
             ENDIF
          INDX(J)=IMAX
          IF (J.NE.N) THEN
             IF (A(J,J).EQ.0) THEN
                A(J,J)=TINY
                IER=2
C IF THE PIVOT ELEMENT IS ZERO THE MATRIX IS SINGULAR (AT LEAST
C TO THE PRECISION OF THE ALGORITHM). FOR SOME APPLICATIONS
C ON SINGULAR MATRICES, IT IS DESIRABLE TO SUBSTITUTE TINY FOR ZERO
                ENDIF
             DUM=1.0D0/A(J,J)
             DO 18 I=J+1,N
                A(I,J)=A(I,J)*DUM
   18           CONTINUE
             ENDIF
   19     CONTINUE
        IF (A(N,N).EQ.0.0D0)A(N,N)=TINY
        RETURN
        END
 
        SUBROUTINE LUBKSB(A,N,NP,INDX,B)
        IMPLICIT NONE
c
c Solves the set of lineair equations A.X=B . Here  A is input, not
c as the matrix A but rather as its LU decomposition, determined
c by the routine LUDCMP. INDX is input as permutation vector returned
c by LUDCMP. B is input as the right-hand side vector B and returns with
c the solution vector X. A,N,NP and INDX are not modified by this routine
c and can be left in place for successive calls with different right-hand
c sides B. This routine takes into account the possibility, that B will
c with many zero's, so it is efficient for use in matrix inversion.
c
        INTEGER N,NP
        REAL*8 A(NP,N),B(N)
        INTEGER INDX(N)
        REAL*8 SUM
        INTEGER I,II,LL,J
        II=0
        DO 12 I=1,N
           LL=INDX(I)
           SUM=B(LL)
           B(LL)=B(I)
           IF (II.NE.0) THEN
              DO 11 J=II,I-1
                 SUM=SUM-A(I,J)*B(J)
   11            CONTINUE
           ELSE IF (SUM.NE.0.0D0) THEN
              II=I
              ENDIF
           B(I)=SUM
   12      CONTINUE
        DO 14 I=N,1,-1
           SUM=B(I)
           IF (I.LT.N) THEN
              DO 13 J=I+1,N
                 SUM=SUM-A(I,J)*B(J)
   13            CONTINUE
              ENDIF
           B(I)=SUM/A(I,I)
   14      CONTINUE
        RETURN
        END
 
        REAL*8 FUNCTION RAN1(IDUM)
        IMPLICIT NONE
        INTEGER IDUM

c
c returns a uniform random deviate between 0.0 and 1.0
c Set IDUM to any negative value to indicate or reinitialize
c the sequence.
c this is the 'heaviest' random function in Numerical Recipes.
c the random numbers al independent and all bits are random
c
        REAL*8 R,RM1,RM2
        INTEGER IFF,IX1,IX2,IX3
        INTEGER M1,IA1,IC1,M2,IA2,IC2,M3,IA3,IC3
        SAVE R,IFF,IX1,IX2,IX3
        DIMENSION R(97)
        PARAMETER (M1=259200,IA1=7141,IC1=54773,RM1=1.0D0/M1)
        PARAMETER (M2=134456,IA2=8121,IC2=28411,RM2=1.0D0/M2)
        PARAMETER (M3=243000,IA3=4561,IC3=51349)
        DATA IFF /0/
        INTEGER J
        IF (IDUM.LT.0 .OR. IFF.EQ.0) THEN
           IFF=1
C       SEED THE FIRST ROUTINE
           IX1=IABS(MOD(IC1-IDUM,M1))
           IX1=MOD(IA1*IX1+IC1,M1)
C       AND USE IT TO SEED THE SECOND
           IX2=MOD(IX1,M2)
           IX1=MOD(IA1*IX1+IC1,M1)
C       AND THIRD ROUTINES
           IX3=MOD(IX1,M3)
           DO 11 J=1,97
C       FILL THE TABLE WITH SEQUENTIAL UNIFORM DEVIATES GENERATED BY
C       THE FIRST
              IX1=MOD(IA1*IX1+IC1,M1)
              IX2=MOD(IA2*IX2+IC2,M2)
              R(J)=(DBLE(IX1)+DBLE(IX2)*RM2)*RM1
C       LOW AND HIGH ORDER PIECES COMBINED HERE
   11         CONTINUE
C           IDUM=1
C COMMENT TO ALLOW CONSTANT FOR IDUM
           ENDIF
C
        IX1=MOD(IA1*IX1+IC1,M1)
        IX2=MOD(IA2*IX2+IC2,M2)
        IX3=MOD(IA3*IX3+IC3,M3)
C       USE THE THIRD SEQUENCE TO GET AN INTEGER BETWEEN 1 AND 97
        J=1+(97*IX3)/M3
        IF(J.GT.97 .OR. J.LT.1) THEN
            PRINT *,'THIS SHOULD NOT HAPPEN !!!, ROUTINE RAN1'
            PRINT *,'J= ',J
            STOP
            ENDIF
        RAN1=R(J)
C       AND REFILL IT
        R(J)=(DBLE(IX1)+DBLE(IX2)*RM2)*RM1
        RETURN
        END
 
        REAL*8 FUNCTION RAN2(IDUM)
        IMPLICIT NONE
        INTEGER IDUM
c returns a uniform random deviate between 0.0 and 1.0.
c set IDUM to any negative value to initialize or reinitialize
c the sequence.
c this is the second heaviest randum number routine in Numerical
c Recipes. The least sgnificant bits are less random than the others.
c Otherwize: values are independent
c
        INTEGER IFF,IY,JDUM,M,IA,IC
        REAL*8 RM
        SAVE IFF,IR,IY,JDUM
        PARAMETER(M=714025,IA=1366,IC=150889,RM=1.0D0/M)
        INTEGER IR(97)
        DATA IFF/0/
        INTEGER J
        IF(IDUM.LT.0 .OR. IFF.EQ.0) THEN
           IFF=1
C       INTRODUCE JDUM TO ALLOW CONSTANT FOR PARAMETER IDUM
           JDUM=IABS(MOD(IC-IDUM,M))
C       INITIALIZE THE SHUFFLE TABLE
           DO 11 J=1,97
              JDUM=MOD(IA*JDUM+IC,M)
              IR(J)=JDUM
   11         CONTINUE
           JDUM=MOD(IA*JDUM+IC,M)
           IY=JDUM
           ENDIF
 
        J=1+(97*IY)/M
        IF(J.GT.97 .OR. J.LT.1) THEN
           PRINT *,'THIS SHOULD NOT HAPPEN !!!! FUNCTION RAN2'
           STOP
           ENDIF
        IY=IR(J)
        RAN2=IY*RM
        JDUM=MOD(IA*JDUM+IC,M)
        IR(J)=JDUM
        RETURN
        END
 
 
        SUBROUTINE SORT(RA,N)
        IMPLICIT NONE
 
C Sorts an array RA of length N into ascending numerical order using
C the Heapsort algorithm. N is input; RA is replaced on output by its
C sorted rearrangement.
C

        INTEGER N
        REAL*8 RA(N)
        INTEGER L,IR,J,I
        REAL*8 RRA
        L=N/2+1
        IR=N
C
C The index L will be decremented from its initial value down to 1
C during the 'hiring' (heap creation) phase. Once it reaches 1,
C the index IR will be decremented from its initial value down
C to 1 during the 'retirement-and-promotion' (heap selection)
C phase.
C
   10   CONTINUE
        IF (L.GT.1)THEN
           L=L-1
           RRA=RA(L)
          ELSE
           RRA=RA(IR)
           RA(IR)=RA(1)
           IR=IR-1
           IF(IR.EQ.1)THEN
              RA(1)=RRA
              RETURN
              ENDIF
           ENDIF
        I=L
        J=L+L
   20   IF(J.LE.IR)THEN
           IF(J.LT.IR)THEN
              IF(RA(J).LT.RA(J+1))J=J+1
              ENDIF
           IF(RRA.LT.RA(J))THEN
              RA(I)=RA(J)
              I=J
              J=J+J
             ELSE
              J=IR+1
              ENDIF
           GOTO 20
           ENDIF
        RA(I)=RRA
        GOTO 10
        END
 
 
c ATARI version of CPTIME:
 
c      FUNCTION CPTIME()
c      DOUBLE PRECISION CPTIME
c      INTEGER*4 SUPER
c      PARAMETER (SUPER     = Z'00000902')
c      INTEGER*4 STACK,ATARI,TIMER
c      STACK=0
c      STACK=ATARI(SUPER,STACK)
c      TIMER=LONG(1210)
c      STACK=ATARI(SUPER,STACK)
c      CPTIME=TIMER*0.005D0
c      RETURN
c      END
 
 
c VAX VMS version of CPTIME:
C
C     FUNCTION CPTIME()
C     DOUBLE PRECISION CPTIME
C     SAVE IFF,TIMER_ADDR
C     INTEGER*4 TIMER_ADDR,STATUS,IFF,ITIME,LIB$STAT_TIMER,TWEE
C     DATA IFF /0/,TWEE /2/
C
C     IF (IFF.EQ.0) THEN
C        STATUS=LIB$INIT_TIMER(TIMER_ADDR)
C        IFF=1
C     ENDIF
C
C     STATUS=LIB$STAT_TIMER(TWEE,ITIME,TIMER_ADDR)
C     CPTIME=ITIME*0.01
C     RETURN
C     END
C   
C IBM VERSIE
C
 
c      FUNCTION CPTIME()
c      DOUBLE PRECISION CPTIME,T
c      CALL CPUTIME(T,I)
c      CPTIME=T*1.0E-6
c      RETURN
c      END
C CDC EN CRAY VERSION OF CPTIME
C
C      FUNCTION CPTIME()
C      DOUBLE PRECISION CPTIME
C      CPTIME=SECOND()
C      RETURN
C      END
 
C   SUN VERSIE
c       DOUBLE PRECISION FUNCTION CPTIME()
c       REAL T(2)
c       CPTIME=ETIME(T)
c       RETURN
c       END
C RS6000 AIX versie
C        FUNCTION CPTIME()
C        DOUBLE PRECISION CPTIME
C        CPTIME=0.01D0*MCLOCK()
C        RETURN
C        END
C universele unix versie
        REAL*8 FUNCTION CPTIME()
        IMPLICIT NONE
        EXTERNAL CPUTIME
        REAL*8 CPUTIME
        CPTIME=CPUTIME()
        RETURN
        END
