%
% author: Willem Vermin
%         SARA
%         Kruislaan 415
%         1098SJ Amsterdam
%         the Netherlands
%
%
% e-mail: SONDWIM@HASARA11
%      or WILLEM_VERMIN@SARA.NL
%
%
%
% for IBM mainframes: compile with AUTODBL(DBL4) option
%
%  determines approximately cpu speed.
%
%  To adapt program to different operating systems: adapt
%  function CPTIME(), which must return the cpu-time elapsed,
%  measured in seconds.
%
%  Output is the used cptime and the number of Kilo WEPS
%  (an arbitrary unit) for each case. Furthermore, a weighted
%  mean and a weighted harmonic mean are output.
%
%  Three cases:
%
% -1 fill matrix A(NP,NP) with random numbers: exercizing
%    subroutine calls and simple calculations.
%    Weight: 1.0
%
% -2 solution of linair equations, matrix of coefficients:  A(NP,NP)
%    Weight: 3.0
%
% -3 sorting an array AA(NR)
%    Weight: 3.0
%
%  NP is order of matrix A, NR is length of array AA
%
% NP is maximum dimension of matrix to be used. Use larger value
%    for fast computers.
%
function weps8()
echo off;
NP=2003;
NR=NP*NP;
%
% Weights for three cases:
%
A1=1.0D0;
A2=3.0D0;
A3=3.0D0;

% FACTOR USED FOR PRINTING WARNING ABOUT IMPRECISE RESULTS

TEPS=0.1D0;

A = zeros(NP);
B = zeros(NP);
X = zeros(1,NP);
Y = zeros(1,NP);
V = zeros(1,NP);
AA = zeros(1,NR);
INDX = zeros(1,NP,'int32');
TMIN = -1.0D0;

RAN1(-1);
RAN2(-1);

fprintf('%s\n',' THIS IS PROGRAM WEPS: MEASURES CPU SPEED')
fprintf('%s\n',' MATLAB VERSION')

while true
   while true
      %fprintf('\n%s\n',' ENTER ORDER OF MATRICES TO BE USED, 0 TO STOP')
      N = input('\n ENTER ORDER OF MATRICES TO BE USED, 0 TO STOP','s');
      fprintf('\n');
      N = str2num(N);
      AN=N*1.0d0;
      if N == 0
         break;
      end
      if N>NP || N<0 || N==1
         fprintf('%s%d\n',' ORDER MUST BE IN THE RANGE 2..',NP)
      else
         break;
      end
   end
   if N == 0
      break;
   end

   if TMIN < 0
      % DETERMINE  MINIMUM TIME PER MEASUREMENT:

      TMIN = DETTMI(TMIN);

      % MEASURE APPROXIMATE TIME (TCP) TO MEASURE CPTIME:

      TCP = MEATCP(TMIN);

      % IF NECCESARY, ADJUST TMIN TO BE AT LEAST 100 TIMES TCP 

      TMIN=max([100.0D0*TCP,TMIN,1.0D0]);

      fprintf('%s%12.4f%s\n',' MINIMUM CPU TIME PER MEASUREMENT IS ',TMIN,' SEC')
   end

   NSORT = N*N;

   fprintf('%s\n',' NUMBER CRUNCHING ...');
   [ T1,LOOP1] = TSTRAN(A,N,NP,TCP,TMIN);
   [ T2,LOOP2] = TSTSOL(A,B,X,Y,V,INDX,N,NP,TCP,TMIN);
   [ T3,LOOP3] = TSTSOR(AA,NSORT,TCP,TMIN);

   % DETERMINE IF RESULTS ARE POSSIBLY INACCURATE DUE TO OVERHEAD
   % OF MONITORING CPTIME:
    
   MARK1=' ';
   MARK2=' ';
   MARK3=' ';
   MARKH=' ';

   if TCP > TEPS*T1
      MARK1='*';
   end
   if TCP > TEPS*T2
      MARK2='*';
   end
   if TCP > TEPS*T3
      MARK3='*';
   end

   INAC=(MARK1 ~= ' ' || MARK2 ~= ' ' || MARK3 ~= ' ');

   if INAC
      MARKH='*';
   end


   % C1, C2 AND C3 ARE PROPORTIONAL TO NUMBER OF CALCULATIONS IN
   %  EACH CASE
    
   C1=10*AN^2;
   C2=1.0D0/3*(AN^3+11*AN^2);
   C3=NSORT*log(1.0d0*NSORT);

   %       CALCULATE KILOWEPS IN EACH CASE
    
   W1=C1/T1*1.0D-3;
   W2=C2/T2*1.0D-3;
   W3=C3/T3*1.0D-3;

   fprintf('%s%8d\n',' DECLARED ORDER OF MATRICES ',NP);
   fprintf('%s%8d\n',' ORDER OF MATRICES:         ',N);
   fprintf('%s%8d\n',' DIMENSION OF SORT ARRAY:   ',NSORT);

   %       CALCULATE WEIGHTED HARMONIC MEAN
    
   HW=(A1+A2+A3)/(A1/W1+A2/W2+A3/W3);
    

   fprintf('                               WEIGHT    CPUTIME    REPS    SPEED\n');
   fprintf('                                         (SECONDS)          (KILOWEPS)\n\n');

   form = '%s%9d:%4.1f%12.6f%8d%12.1f    %1s\n';
   firm = '%-56s%12.1f    %1s\n';
   fprintf(form,'SUBROUTINE, # CALLS IS',N*N,  A1,T1,LOOP1,W1,MARK1);
   fprintf(form,'LIN EQUATIONS OF ORDER',N,    A2,T2,LOOP2,W2,MARK2);
   fprintf(form,'SORTING, # ELEMENTS IS',NSORT,A3,T3,LOOP3,W3,MARK3);
   fprintf(firm,'HARMONIC WEIGHTED MEAN',HW,MARKH);
    
   if INAC
      fprintf('* MEANS: POSSIBLY INACCURATE\n');
   end
    
   fprintf('\nEND OF THIS WEPS MEASUREMENT');
end
exit(0);
end % end of function main


function [TMIN] = DETTMI(TMIN)
M=10;
P=100.0D0;

% DETERMINES GRANULARITY OF TIME MEASUREMENT, AND CALCULATES
% TMIN, SO THAT TMIN IS A REASONABLE MINIMUM CPTIME FOR AN ACCURACY
% OF APPROXIMATELY ONE PERCENT

T0=CPTIME();
DT=0.0D0;
for I = 1:M
   while true
      T=CPTIME();
      if T ~= T0
         break;
      end
   end
   DT=DT+T-T0;
   T0=T;
end

TMIN=P*DT/M;

if TMIN>100.0D0
   fprintf('%s\n',' MINIMUM TIME FOR MEASUREMENTS GREATER THAN 100 SEC');
   fprintf('%s\n',' SOMETHING IS PROBABLY WRONG, PROGRAM STOPS');
   exit(1);
end
end  % DETTMI

function [TCP] = MEATCP(TMIN)

LOOP=0;
T0=CPTIME();

% MAKE TCP=0, WITHOUT LETTING THE COMPILER KNOW:

TCP=LOOP*T0;

while true

   LOOP=LOOP+1;
   T=CPTIME();
   DT=T-T0-LOOP*TCP;
   if DT >= TMIN
      break;
   end
end
TCP=TMIN/LOOP;

end % MEATCP

function [t] = CPTIME()
   t = cputime();
end % CPTIME

function [T,LOOP] =  TSTRAN(A,N,NP,TCP,TMIN)
LOOP=0;
T0=CPTIME();
while true
   LOOP=LOOP+1;

   for I=1:N
      for J=1:N
         A(I,J)=RAN1(1);
      end
   end

   T=CPTIME();
   DT=T-T0-LOOP*TCP;
   if(DT >= TMIN)
        break;
   end
end

T=DT/LOOP;
end % TSTRAN

function [RAN] = RAN1(IDUM)

%
% returns a uniform random deviate between 0.0 and 1.0
% Set IDUM to any negative value to indicate or reinitialize
% the sequence.
% this is the 'heaviest' random function in Numerical Recipes.
% the random numbers al independent and all bits are random
%
persistent R IFF IX1 IX2 IX3;
if isempty(R)
   R = zeros(1,97);
   IFF = 0;
end
M1=259200;IA1=7141;IC1=54773;RM1=1.0D0/M1;
M2=134456;IA2=8121;IC2=28411;RM2=1.0D0/M2;
M3=243000;IA3=4561;IC3=51349;

if (IDUM<0 || IFF==0)
   IFF=1;
   %       SEED THE FIRST ROUTINE
   IX1=abs(mod(IC1-IDUM,M1));
   IX1=mod(IA1*IX1+IC1,M1);
   %       AND USE IT TO SEED THE SECOND
   IX2=mod(IX1,M2);
   IX1=mod(IA1*IX1+IC1,M1);
   %       AND THIRD ROUTINES
   IX3=mod(IX1,M3);
   for J=1:97
      %       FILL THE TABLE WITH SEQUENTIAL UNIFORM DEVIATES GENERATED BY
      %       THE FIRST
      IX1=mod(IA1*IX1+IC1,M1);
      IX2=mod(IA2*IX2+IC2,M2);
      R(J)=(1.0d0*IX1+1.0d0*IX2*RM2)*RM1;
      %       LOW AND HIGH ORDER PIECES COMBINED HERE
   end
   %           IDUM=1
   % COMMENT TO ALLOW CONSTANT FOR IDUM
end 
%
IX1=mod(IA1*IX1+IC1,M1);
IX2=mod(IA2*IX2+IC2,M2);
IX3=mod(IA3*IX3+IC3,M3);
%       USE THE THIRD SEQUENCE TO GET AN INTEGER BETWEEN 1 AND 97
J=1+floor((97*IX3)/M3);
if J>97 || J<1
   fprintf('%s\n', 'THIS SHOULD NOT HAPPEN !!!, ROUTINE RAN1')
   fprintf('%s%d\n','J= ',J);
   exit(1);
end
RAN=R(J);
%       AND REFILL IT
R(J)=(IX1*1.0d0+IX2*1.0d0*RM2)*RM1;
end % RAN1

function [RAN] = RAN2(IDUM)
% returns a uniform random deviate between 0.0 and 1.0.
% set IDUM to any negative value to initialize or reinitialize
% the sequence.
% this is the second heaviest randum number routine in Numerical
% Recipes. The least sgnificant bits are less random than the others.
% Otherwize: values are independent
%
        persistent IFF IR IY JDUM;
        M=714025;IA=1366;IC=150889;RM=1.0D0/M;
        if isempty(IR)
           IR = zeros(1,97);
           IFF = 0;
        end
        if IDUM<0 || IFF==0
           IFF=1;
%       INTRODUCE JDUM TO ALLOW CONSTANT FOR PARAMETER IDUM
           JDUM=abs(mod(IC-IDUM,M));
%       INITIALIZE THE SHUFFLE TABLE
           for J=1:97
              JDUM=mod(IA*JDUM+IC,M);
              IR(J)=JDUM;
           end
           JDUM=mod(IA*JDUM+IC,M);
           IY=JDUM;
        end
 
        J=1+floor((97*IY)/M);
        if J>97 || J<1
           fprintf('THIS SHOULD NOT HAPPEN !!!! FUNCTION RAN2\n');
	   fprintf('J IY M: %d %d %d\n',J,IY,M);
           exit(1);
        end
        IY=IR(J);
        RAN=IY*RM;
        JDUM=mod(IA*JDUM+IC,M);
        IR(J)=JDUM;
end % RAN2

function [T,LOOP] = TSTSOL(A,B,X,Y,V,INDX,N,NP,TCP,TMIN)

for  I=1:N
   for J=1:N
      B(I,J)=RAN2(1);
   end
   Y(I)=RAN2(1);
end

LOOP=0;
T0=CPTIME();
while true
   LOOP=LOOP+1;

   for I=1:N
      for J=1:N
         A(I,J)=B(I,J);
      end
      X(I)=Y(I);
   end
   [IER,D,A,INDX] = LUDCMP(A,N,NP);
   if IER ~= 0
      fprintf('%s\n',' MATRIX SINGULAR IN TSTSOL');
      fprintf('%s\n',' PROGAM ENDS HERE');
      exit(1);
   end
   X = LUBKSB(A,N,NP,INDX,X);
   for I=1:N
      S=0.0D0;
      for J=1:N
         S=S+B(I,J)*X(J);
      end
      if abs(S-Y(I))>0.01D0*N
         fprintf('%s\n',' INCORRECT SOLUTION IN TSTSOL');
         fprintf('%s\n',' PROGRAM ENDS HERE');
         exit(1);
      end
   end

   T=CPTIME();
   DT=T-T0-LOOP*TCP;
   if(DT >= TMIN)
      break;
   end
end

T=DT/LOOP;
end % TSTSOL

function [IER, D,A,INDX] =  LUDCMP(A,N,NP)

%
%       see: Numerical Recipes, chapter 2.3
%
%       given an N * N matrix A, with physical dimensions NP, this routine
%       replaces it by the LU decomposition of a rowwise permutation of
%       itself. A and N are input. A is output, arranged as follows:
%       strict under triangle = L, upper triangle = U.
%       VV is a workarray with dimension of at least N.(this is a
%       minor deviation from the original text).
%       INDX is an output vector which records the row permutation
%       effected by the partial pivoting; D is output as +1 or -1
%       depending on whether the number of row interchanges was even
%       or odd, respectively. This routine is used in combination with
%       LUBKSB to solve linear equations or inver a matrix.
%       In case A is singular, IER=1 and D=0.0 further calculations are
%       bypassed.
%       In case A is nearly singular, IER =2.
%       IER=0 if all is ok.
%       The determinant of A is obtained by multiplying the product of the
%       main diagonal elements of the output matrix A with D.
%
VV = zeros(1,N);
INDX = zeros(1,N,'int32');
TINY=1.0D-20;

IMAX=0;
D=1.0D0;
IER=0;
%       LOOP OVER ROWS TO GET THE IMPLICIT SCALING INFORMATION
for  I=1:N
   AAMAX=0.0D0;
   for J=1:N
      if abs(A(I,J))>AAMAX
         AAMAX=abs(A(I,J));
      end
   end
   if (AAMAX == 0.0D0)
      IER=1;
      D=0.0D0;
      return;
   end
   VV(I)=1.0D0/AAMAX;
end
% THIS IS THE LOOP OVER COLUMS OF CROUT'S METHOD
for J=1:N
   if J>1
      for I=1:J-1
        SUM=A(I,J);
        if I>1
           for K=1:I-1
              SUM=SUM-A(I,K)*A(K,J);
           end
           A(I,J)=SUM;
        end
     end
  end
% INITIALIZE FOR THE SEARCH OF THE LARGEST PIVOT ELEMENT
  AAMAX=0.0D0;
  for I=J:N
     SUM=A(I,J);
     if (J>1)
        for K=1:J-1
           SUM=SUM-A(I,K)*A(K,J);
        end
        A(I,J)=SUM;
     end
% FIGURE OF MERIT FOR THE PIVOT
     DUM=VV(I)*abs(SUM);
     if DUM>AAMAX
        IMAX=I;
        AAMAX=DUM;
     end
  end
% INTERCHANGE ROWS IF NECESSARY
  if J~=IMAX
     for K=1:N
        DUM=A(IMAX,K);
        A(IMAX,K)=A(J,K);
        A(J,K)=DUM;
     end
     D=-D;
     VV(IMAX)=VV(J);
  end
  INDX(J)=IMAX;
  if J~=N
     if A(J,J)==0;
        A(J,J)=TINY;
        IER=2;
% IF THE PIVOT ELEMENT IS ZERO THE MATRIX IS SINGULAR (AT LEAST
% TO THE PRECISION OF THE ALGORITHM). FOR SOME APPLICATIONS
% ON SINGULAR MATRICES, IT IS DESIRABLE TO SUBSTITUTE TINY FOR ZERO
      end
      DUM=1.0D0/A(J,J);
      for I=J+1:N
         A(I,J)=A(I,J)*DUM;
      end
   end
end
if A(N,N)==0.0D0
   A(N,N)=TINY;
end
end % LUDCMP

function [B] =  LUBKSB(A,N,NP,INDX,B)
%
% Solves the set of lineair equations A.X=B . Here  A is input, not
% as the matrix A but rather as its LU decomposition, determined
% by the routine LUDCMP. INDX is input as permutation vector returned
% by LUDCMP. B is input as the right-hand side vector B and returns with
% the solution vector X. A,N,NP and INDX are not modified by this routine
% and can be left in place for successive calls with different right-hand
% sides B. This routine takes into account the possibility, that B will
% with many zero's, so it is efficient for use in matrix inversion.
%
II=0;
for I=1:N
   LL=INDX(I);
   SUM=B(LL);
   B(LL)=B(I);
   if II~=0
      for J=II:I-1
         SUM=SUM-A(I,J)*B(J);
      end
   elseif SUM~=0.0D0
      II=I;
   end
   B(I)=SUM;
end
for I=N:-1:1
   SUM=B(I);
   if I<N
      for J=I+1:N
         SUM=SUM-A(I,J)*B(J);
      end
   end
   B(I)=SUM/A(I,I);
end
end % LUBKSB

function [T,LOOP] = TSTSOR(A,N,TCP,TMIN)

for I=1:N
   A(I)=RAN2(1);
end

% SORTING, SO ALL SORTS ARE EQUAL LATER ON
A = SORT(A,N);
LOOP=0;
T0=CPTIME();

while true

   LOOP=LOOP+1;

   % SORT USES HEAPSORT ALGORITHM: IT MAKES NO DIFFERENCE IF ARRAY IS
   % ALREADY SORTED

   SORT(A,N);

   T=CPTIME();
   DT=T-T0-LOOP*TCP;
   if(DT >= TMIN)
      break;
   end
end

T=DT/LOOP;

LLL=true;
for I=1:N-1
   LLL=LLL & (A(I)<=A(I+1));
end
if (~LLL)
   fprintf('%s\n',' ARRAY NOT SORTED IN TSTSOR');
   fprintf('%s\n',' PROGRAM ENDS HERE');
   exit(1);
end
end % TSTSOR

function [RA] = SORT(RA,N)

% Sorts an array RA of length N into ascending numerical order using
% the Heapsort algorithm. N is input; RA is replaced on output by its
% sorted rearrangement.
%

L=floor(N/2)+1;
IR=N;
%
% The index L will be decremented from its initial value down to 1
% during the 'hiring' (heap creation) phase. Once it reaches 1,
% the index IR will be decremented from its initial value down
% to 1 during the 'retirement-and-promotion' (heap selection)
% phase.
%
while true
   if L>1
      L=L-1;
      RRA=RA(L);
   else
      RRA=RA(IR);
      RA(IR)=RA(1);
      IR=IR-1;
      if IR==1
         RA(1)=RRA;
         return;
      end
   end
   I=L;
   J=L+L;
   while J <= IR
      if J<IR
         if RA(J)<RA(J+1)
            J=J+1;
         end
      end
      if RRA<RA(J)
         RA(I)=RA(J);
         I=J;
         J=J+J;
      else
         J=IR+1;
      end
   end
   RA(I)=RRA;
end
end  % SORT
