#       PROGRAM WEPS
#
# author: Willem Vermin
#         SARA
#         Kruislaan 415
#         1098SJ Amsterdam
#         the Netherlands
#
#
# e-mail: SONDWIM@HASARA11
#      or WILLEM_VERMIN@SARA.NL
#
#
#
#  determines approximately cpu speed.
#
#  Output is the used cptime and the number of Kilo WEPS
#  (an arbitrary unit) for each case. Furthermore, a weighted
#  mean and a weighted harmonic mean are output.
#
#  Three cases:
#
# -1 fill matrix A(NP,NP) with random numbers: exercizing
#    subroutine calls and simple calculations.
#    Weight: 1.0
#
# -2 solution of linair equations, matrix of coefficients:  A(NP,NP)
#    Weight: 3.0
#
# -3 sorting an array AA(NR)
#    Weight: 3.0
#
#  NP is order of matrix A, NR is length of array AA
#
# NP is maximum dimension of matrix to be used. Use larger value
#    for fast computers.
#

import time, sys
from math import log

def CPTIME():
    return time.clock()

def MEATCP(TMIN):

    LOOP=0
    T0=CPTIME()

    # MAKE TCP=0, WITHOUT LETTING THE COMPILER KNOW:

    TCP=LOOP*T0

    while True:

        LOOP=LOOP+1
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        if DT >= TMIN:
            break

    TCP=TMIN/LOOP

    return TCP
 
def DETTMI():
    M=10
    P=100.0e0
 
# DETERMINES GRANULARITY OF TIME MEASUREMENT, AND CALCULATES
# TMIN, SO THAT TMIN IS A REASONABLE MINIMUM CPTIME FOR AN ACCURACY
# OF APPROXIMATELY ONE PERCENT
 
    T0=CPTIME()
    DT=0.0
    for I in range(M):
        while True:
            T=CPTIME()
            if T != T0:
                break
    DT=DT+T-T0
    T0=T
 
    TMIN=P*DT/M
 
    if TMIN>100.0:
          print ' MINIMUM TIME FOR MEASUREMENTS GREATER THAN 100 SEC'
          print ' SOMETHING IS PROBABLY WRONG, PROGRAM STOPS'
          sys.exit(1)
    return TMIN
 
def TSTRAN(A,N,NP,TCP,TMIN):
 
    LOOP=0
    T0=CPTIME()
    while True:
        LOOP=LOOP+1
        for I in range(N):
            for J in range(N):
                 A[I][J]=RAN1(1)
 
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        if DT >= TMIN:
            break
 
    T=DT/LOOP
 
    return T,LOOP

def TSTSOL(A,B,X,Y,V,INDX,N,NP,TCP,TMIN):
 
    for I in range(N):
        for J in range(N):
            B[I][J] = RAN2(1)
        Y[I] = RAN2(1)

        LOOP=0
        T0=CPTIME()
    while True:
        LOOP=LOOP+1
         
        for I in range(N):
            for J in range(N):
                A[I][J] = B[I][J]
            X[I] = Y[I]
        IER, D = LUDCMP(A,N,NP,INDX,V)
        if IER != 0:
            print ' MATRIX SINGULAR IN TSTSOL'
            print ' PROGAM ENDS HERE'
            sys.exit(1)

        LUBKSB(A,N,NP,INDX,X)

        for I in range(N):
             S=0.0
             for J in range(N):
                 S=S+B[I][J]*X[J]
             if abs(S-Y[I])>0.01*N:
                 print ' INCORRECT SOLUTION IN TSTSOL'
                 print ' PROGRAM ENDS HERE'
                 sys.exit(1)
 
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        if DT >= TMIN:
            break
 
    T=DT/LOOP
 
    return T,LOOP

def TSTSOR(A,N,TCP,TMIN):

    for I in range(N):
        A[I]=RAN2(1)
 
# SORTING, SO ALL SORTS ARE EQUAL LATER ON
    SORT(A,N)
    LOOP=0
    T0=CPTIME()
 
    while True:
 
        LOOP=LOOP+1
 
# SORT USES HEAPSORT ALGORITHM: IT MAKES NO DIFFERENCE IF ARRAY IS
# ALREADY SORTED
 
        SORT(A,N)
 
        T=CPTIME()
        DT=T-T0-LOOP*TCP
        if DT >= TMIN:
            break
 
    T=DT/LOOP

    LLL=True
    for I in range(N-1):
         LLL=LLL and (A[I]<=A[I+1])

    if not LLL:
        print ' ARRAY NOT SORTED IN TSTSOR'
        print ' PROGRAM ENDS HERE'
        sys.exit(1)
 
    return T,LOOP

def LUDCMP(A,N,NP,INDX,VV):
#
#       see: Numerical Recipes, chapter 2.3
#
#       given an N * N matrix A, with physical dimensions NP, this routine
#       replaces it by the LU decomposition of a rowwise permutation of
#       itself. A and N are input. A is output, arranged as follows:
#       strict under triangle = L, upper triangle = U.
#       VV is a workarray with dimension of at least N.(this is a
#       minor deviation from the original text).
#       INDX is an output vector which records the row permutation
#       effected by the partial pivoting; D is output as +1 or -1
#       depending on whether the number of row interchanges was even
#       or odd, respectively. This routine is used in combination with
#       LUBKSB to solve linear equations or inver a matrix.
#       In case A is singular, IER=1 and D=0.0 further calculations are
#       bypassed.
#       In case A is nearly singular, IER =2.
#       IER=0 if all is ok.
#       The determinant of A is obtained by multiplying the product of the
#       main diagonal elements of the output matrix A with D.
#
        TINY=1.0e-20

        IMAX=0
        D=1.0
        IER=0
#       LOOP OVER ROWS TO GET THE IMPLICIT SCALING INFORMATION
        for I in range(N):
           AAMAX=0.0
           for J in range(N):
               if abs(A[I][J])>AAMAX:
                   AAMAX=abs(A[I][J])

           if AAMAX==0.0:
              IER=1
              D=0.0
              return IER, D

           VV[I]=1.0/AAMAX

# THIS IS THE LOOP OVER COLUMS OF CROUT'S METHOD
       
        for J in range(N):
          if J > 0:
             for I in range(J):
                SUM=A[I][J]
                if I > 0:
                   for K in range(I):
                      SUM=SUM-A[I][K]*A[K][J]
                   A[I][J]=SUM
# INITIALIZE FOR THE SEARCH OF THE LARGEST PIVOT ELEMENT
          AAMAX=0.0
          for I in range(J,N):
             SUM=A[I][J]
             if J > 0:
                for K in range(J):
                   SUM=SUM-A[I][K]*A[K][J]
                A[I][J]=SUM
# FIGURE OF MERIT FOR THE PIVOT
             DUM=VV[I]*abs(SUM)
             if DUM > AAMAX:
                IMAX=I
                AAMAX=DUM
# INTERCHANGE ROWS IF NECESSARY
          if J != IMAX:
             for K in range(N):
                DUM=A[IMAX][K]
                A[IMAX][K]=A[J][K]
                A[J][K]=DUM
             D=-D
             VV[IMAX]=VV[J]
          INDX[J]=IMAX
          if J != N:
              if A[J][J] == 0:
                A[J][J]=TINY
                IER=2
# IF THE PIVOT ELEMENT IS ZERO THE MATRIX IS SINGULAR (AT LEAST
# TO THE PRECISION OF THE ALGORITHM). FOR SOME APPLICATIONS
# ON SINGULAR MATRICES, IT IS DESIRABLE TO SUBSTITUTE TINY FOR ZERO
              DUM=1.0/A[J][J]
              for I in range(J+1,N):
                A[I][J]=A[I][J]*DUM
        if A[N][N]==0.0:
           A[N][N]=TINY

        return IER,D
   
 
 
def LUBKSB(A,N,NP,INDX,B):
#
# Solves the set of lineair equations A.X=B . Here  A is input, not
# as the matrix A but rather as its LU decomposition, determined
# by the routine LUDCMP. INDX is input as permutation vector returned
# by LUDCMP. B is input as the right-hand side vector B and returns with
# the solution vector X. A,N,NP and INDX are not modified by this routine
# and can be left in place for successive calls with different right-hand
# sides B. This routine takes into account the possibility, that B will
# with many zero's, so it is efficient for use in matrix inversion.
#
        II=-1
        for I in range(N):
           LL=INDX[I]
           SUM=B[LL]
           B[LL]=B[I]
           if II != -1:
              for J in range(II,I): 
                 SUM=SUM-A[I][J]*B[J]
           elif SUM != 0.0:
              II=I
           B[I]=SUM

        for I in range(N-1,-1,-1):
           SUM=B[I]
           if  I < N:
              for J in range(I+1,N):
                 SUM=SUM-A[I][J]*B[J]
           B[I]=SUM/A[I][I]
        return
 
def SORT(RA,N):
 
# Sorts an array RA of length N into ascending numerical order using
# the Heapsort algorithm. N is input; RA is replaced on output by its
# sorted rearrangement.
#

    L=N/2
    IR=N-1
#
# The index L will be decremented from its initial value down to 1
# during the 'hiring' (heap creation) phase. Once it reaches 1,
# the index IR will be decremented from its initial value down
# to 1 during the 'retirement-and-promotion' (heap selection)
# phase.
#
    while True:
        if L>0:
           L=L-1
           RRA=RA[L]
        else:
           RRA=RA[IR]
           RA[IR]=RA[0]
           IR=IR-1
           if IR == 0:
              RA[0]=RRA
              return
        I=L
        J=L+L+1
        while J <= IR:
            if J < IR:
                if RA[J]<RA[J+1]:
                    J=J+1
            if RRA<RA[J]:
              RA[I]=RA[J]
              I=J
              J=J+J+1
            else:
              J=IR+1
        RA[I]=RRA

class ran1:
    def __init__(self):
        self.M1=259200
        self.IA1=7141
        self.IC1=54773
        self.RM1=1.0/self.M1
        self.M2=134456
        self.IA2=8121
        self.IC2=28411
        self.RM2=1.0/self.M2
        self.M3=243000
        self.IA3=4561
        self.IC3=51349
        self.R = [0.0 for i in range(97)]
        self.IFF = 0

    def RANDOM(self,IDUM):

    #
    # returns a uniform random deviate between 0.0 and 1.0
    # Set IDUM to any negative value to indicate or reinitialize
    # the sequence.
    # this is the 'heaviest' random function in Numerical Recipes.
    # the random numbers al independent and all bits are random
    #
            if  IDUM<0 or self.IFF==0:
               self.IFF=1
    #       SEED THE FIRST ROUTINE
               self.IX1=abs((self.IC1-IDUM)%self.M1)
               self.IX1=(self.IA1*self.IX1+self.IC1)%self.M1
    #       AND USE IT TO SEED THE SECOND
               self.IX2=self.IX1%self.M2
               self.IX1=(self.IA1*self.IX1+self.IC1)%self.M1
    #       AND THIRD ROUTINES
               self.IX3=self.IX1%self.M3
               for J in range(97):
    #       FILL THE TABLE WITH SEQUENTIAL UNIFORM DEVIATES GENERATED BY
    #       THE FIRST
                  self.IX1=(self.IA1*self.IX1+self.IC1)%self.M1
                  self.IX2=(self.IA2*self.IX2+self.IC2)%self.M2
                  self.R[J]=(1.0*self.IX1+1.0*self.IX2*self.RM2)*self.RM1
    #       LOW AND HIGH ORDER PIECES COMBINED HERE
    #          IDUM=1
    # COMMENT TO ALLOW CONSTANT FOR IDUM
    #
            self.IX1=(self.IA1*self.IX1+self.IC1)%self.M1
            self.IX2=(self.IA2*self.IX2+self.IC2)%self.M2
            self.IX3=(self.IA3*self.IX3+self.IC3)%self.M3
    #       USE THE THIRD SEQUENCE TO GET AN INTEGER BETWEEN 1 AND 97
            J=(97*self.IX3)/self.M3
            if J>96 or J<0:
                print 'THIS SHOULD NOT HAPPEN !!!, ROUTINE ran1.RANDOM'
                print 'J= ',J
                sys.exit(1)
            RAN=self.R[J]
    #       AND REFILL IT
            self.R[J]=(1.0*self.IX1+1.0*self.IX2*self.RM2)*self.RM1
            return RAN

class ran2():
    def __init__(self):
        self.IFF = 0
        self.IR = [0 for i in range(97)]
        self.M=714025
        self.IA=1366
        self.IC=150889
        self.RM=1.0/self.M


    def RANDOM(self,IDUM):
    # returns a uniform random deviate between 0.0 and 1.0.
    # set IDUM to any negative value to initialize or reinitialize
    # the sequence.
    # this is the second heaviest randum number routine in Numerical
    # Recipes. The least sgnificant bits are less random than the others.
    # Otherwize: values are independent
    #
            #global IFF,IR,IY,JDUM
            if IDUM<0 or self.IFF==0:
               self.IFF=1
    #       INTRODUCE JDUM TO ALLOW CONSTANT FOR PARAMETER IDUM
               self.JDUM=abs((self.IC-IDUM)%self.M)
    #       INITIALIZE THE SHUFFLE TABLE
               for J in range(97):
                  self.JDUM=(self.IA*self.JDUM+self.IC)%self.M
                  self.IR[J]=self.JDUM
               self.JDUM=(self.IA*self.JDUM+self.IC)%self.M
               self.IY=self.JDUM
     
            J=(97*self.IY)/self.M
            if J>96 or J<0:
               print 'THIS SHOULD NOT HAPPEN !!!! FUNCTION ran2.RANDOM'
               sys.exit(1)
            self.IY=self.IR[J]
            RAN=self.IY*self.RM
            self.JDUM=(self.IA*self.JDUM+self.IC)%self.M
            self.IR[J]=self.JDUM
            return RAN
 
def endprogram():
    print ' END OF PROGRAM WEPS'
    sys.exit(0)

    
 
# main program

NP=2003
NR=NP*NP
#
# Weights for three cases:
#
A1=1.0
A2=3.0
A3=3.0
 
# FACTOR USED FOR PRINTING WARNING ABOUT IMPRECISE RESULTS
 
TEPS=0.1
 
#        REAL*8 A(NP,NP),B(NP,NP),X(NP),Y(NP),V(NP),AA(NR)
A = [[0.0 for i in range(NP)] for j in range(NP)]
B = [[0.0 for i in range(NP)] for j in range(NP)]
X = [0.0 for i in range(NP)]
Y = [0.0 for i in range(NP)]
V = [0.0 for i in range(NP)]
AA = [0.0 for i in range(NR)]

INDX = NP*[0]
TMIN = -1.0
 
N = NP
 
RAN1 = ran1().RANDOM
RAN2 = ran2().RANDOM
RAN1(-1)
RAN2(-1)

print ' THIS IS PROGRAM WEPS: MEASURES CPU SPEED'
print ' PYTHON VERSION'
while True:
    while True:
        print
        print ' ENTER ORDER OF MATRICES TO BE USED, 0 TO STOP'
        try:
            N = int(raw_input())
            AN = N*1.0
        except:
            endprogram()
        if N == 0:
            endprogram()

        if  N > NP or N < 0 or N == 1:
               print ' ORDER MUST BE IN THE RANGE 2..' ,NP
        else:
           break
     
    # DETERMINE  MINIMUM TIME PER MEASUREMENT:
    if  TMIN < 0:
        TMIN = DETTMI()
     
        # MEASURE APPROXIMATE TIME (TCP) TO MEASURE CPTIME:
         
        TCP = MEATCP(TMIN)
         
        # IF NECCESARY, ADJUST TMIN TO BE AT LEAST 100 TIMES TCP 
         
        TMIN=max(100.0*TCP,TMIN,1.0)
     
    print ' MINIMUM CPU TIME PER MEASUREMENT IS ',TMIN,' SEC'
     
    NSORT=N*N
     
    print ' NUMBER CRUNCHING ...'
     
    T1,LOOP1 = TSTRAN(A,N,NP,TCP,TMIN)
    T2,LOOP2 = TSTSOL(A,B,X,Y,V,INDX,N,NP,TCP,TMIN)
    T3,LOOP3 = TSTSOR(AA,NSORT,TCP,TMIN)
     
    # DETERMINE IF RESULTS ARE POSSIBLY INACCURATE DUE TO OVERHEAD
    # OF MONITORING CPTIME:
     
    MARK1=' '
    MARK2=' '
    MARK3=' '
    MARKH=' '
     
    if TCP > TEPS*T1:
        MARK1='*'
    if TCP > TEPS*T2:
        MARK2='*'
    if TCP > TEPS*T3:
        MARK3='*'
     
    INAC= MARK1 != ' ' or MARK2 != ' ' or MARK3 != ' '
     
    if INAC:
        MARKH='*'
     
    # C1, C2 AND C3 ARE PROPORTIONAL TO NUMBER OF CALCULATIONS IN
    #  EACH CASE
     

    C1=10.0*AN**2
    C2=1.0/3*(AN**3+11*AN**2)
    C3=NSORT*log(NSORT)
     
    #       CALCULATE KILOWEPS IN EACH CASE
     
    W1=C1/T1*1.0e-3
    W2=C2/T2*1.0e-3
    W3=C3/T3*1.0e-3
     
    print ' DECLARED ORDER OF MATRICES ',NP
    print ' ORDER OF MATRICES:         ',N
    print ' DIMENSION OF SORT ARRAY:   ',NSORT
     
    #       CALCULATE WEIGHTED HARMONIC MEAN
     
    HW=(A1+A2+A3)/(A1/W1+A2/W2+A3/W3)
     
    form = '{0:6d}:{1:4.1f}{2:14.6f}{3:10d}{4:12.1f}    {5:1s}'
    firm = '      :                            {0:12.1f}    {1:1s}'
    print
    print 30*' ',' WEIGHT    CPUTIME','     REPS','     SPEED'
    print 40*' ','(SECONDS)','           (KILOWEPS)'
    print
    print ' SUBROUTINE, # CALLS IS',form.format(N*N,A1,T1,LOOP1,W1,MARK1)
    print ' LIN EQUATIONS OF ORDER',form.format(N,A2,T2,LOOP2,W2,MARK2)
    print ' SORTING, # ELEMENTS IS',form.format(NSORT,A3,T3,LOOP3,W3,MARK3)
    print ' HARMONIC WEIGHTED MEAN',firm.format(HW,MARKH)
     
    if INAC: 
         print ' * MEANS: POSSIBLY INACCURATE'
     
     
    print
    print '  END OF THIS WEPS MEASUREMENT'
 
